﻿using Exam.Infrastructure.Data.Entity;
using Exam.Share.ViewResponses.Exam;
using Exam.Share.ViewResponses.Question;
using Hotline.Application.Exam.Constants.ApiRoutes;
using Hotline.Application.Exam.Interface.ExamManages;
using Hotline.Application.Exam.Service.ExamManages;
using Hotline.Application.ExportExcel;
using Hotline.Exams.ExamManages;
using Hotline.Exams.Questions;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Requests.Exam;
using Hotline.Share.Requests.Question;
using Hotline.Share.ViewResponses.Exam;
using Hotline.Tools;
using MapsterMapper;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;

namespace Hotline.Api.Controllers.Exam
{
    public class OfflineExamAnalysisController : BaseController
    {
        private readonly IOfflineExamAnalysisService _offineExamAnalysisService;
        private readonly IMapper _mapper;
        private readonly IExportApplication _exportApplication;

        public OfflineExamAnalysisController(IOfflineExamAnalysisService offineExamAnalysisService,IMapper mapper, IExportApplication exportApplication)
        {
            _offineExamAnalysisService = offineExamAnalysisService;
            this._mapper = mapper;
            this._exportApplication = exportApplication;
        }

        /// <summary>
        /// 获取线下考试分页列表
        /// </summary>
        /// <param name="offlineExamAnalysisPagedRequest"></param>
        /// <returns></returns>
        [HttpPost(OfflineExamAnalysisApiRoute.GetPagedList)]
        public async Task<PageViewResponse<OfflineExamAnalysisViewResponse>> GetPagedList([FromBody] OfflineExamAnalysisPagedRequest offlineExamAnalysisPagedRequest)
        {
            var offlineExamAnalysisPageViewResponse = await _offineExamAnalysisService.GetPagedListAsync(offlineExamAnalysisPagedRequest);            

            return offlineExamAnalysisPageViewResponse;
        }

        /// <summary>
        /// 导入Excel
        /// </summary>
        /// <returns></returns>
        [HttpPost(OfflineExamAnalysisApiRoute.ImportExcel)]
        public async Task ImportExcel(IFormFile file)
        {
            await _offineExamAnalysisService.ImportExcel(file, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 导出线下考试成绩
        /// </summary>
        /// <param name="exportExcelDto"></param>
        /// <returns></returns>
        [HttpPost(OfflineExamAnalysisApiRoute.ExportOfflineExamAnalysis)]
        public async Task<FileStreamResult> ExportOfflineExamAnalysis([FromBody] ExportExcelDto<OfflineExamAnalysisPagedRequest> exportExcelDto)
        {
            if (exportExcelDto.IsExportAll)
            {
                exportExcelDto.QueryDto.IsPaged = false;
            }
            var result = await _offineExamAnalysisService.GetPagedListAsync(exportExcelDto.QueryDto);

            dynamic? dynamicClass = DynamicClassHelper.CreateDynamicClass<OfflineExamAnalysisViewResponse>(exportExcelDto.ColumnInfos ?? new List<ColumnInfo>());

            var dtos = result.Items
                .Select(stu => _mapper.Map(stu, typeof(OfflineExamAnalysisViewResponse), dynamicClass))
                .Cast<object>()
                .ToList();

            var stream = ExcelHelper.CreateStream(dtos);

            return ExcelStreamResult(stream, "导出线下考试成绩");

        }

        /// <summary>
        /// 下载Excel模版
        /// </summary>
        /// <returns></returns>
        [HttpGet(OfflineExamAnalysisApiRoute.Download)]
        public FileStreamResult Download()
        {
            var list = new List<OfflineExamAnalysisExcel>();
            return _exportApplication.ExportData(list, "线下考试成绩统计表.xlsx");
        }
    }
}
